/// <reference types="react" />
import { CalendarViewType, WeekDayType } from './models';
export interface UseCalendarOptions {
    defaultDate?: Date | number | string;
    defaultWeekStart?: WeekDayType;
    defaultViewType?: CalendarViewType;
}
export default function useCalendar(options?: UseCalendarOptions): {
    calendar: {
        viewType: CalendarViewType;
        date: Date;
        year: number;
        month: number;
        day: number;
        weekStartsOn: WeekDayType;
        startWeekdayInMonth: number;
        weeksInMonth: number;
        weekendDays: Date[];
        getTargetDate: (weekIndex: number, dayIndex: number) => Date;
        getCurrentWeek: () => number;
        getWeek: (weekIndex: number) => {
            date: Date;
            isCurrentMonth: boolean;
            isCurrentDate: boolean;
        }[];
        getMonth: () => {
            date: Date;
            isCurrentMonth: boolean;
            isCurrentDate: boolean;
        }[][];
        isCurrentMonth: (targetDate: Date) => boolean;
        isCurrentDate: (targetDate: Date) => boolean;
    };
    headers: {
        weekDays: {
            key: string;
            value: Date;
        }[];
    };
    body: {
        weeks: {
            key: string;
            value: {
                key: string;
                value: {
                    date: Date;
                    isCurrentMonth: boolean;
                    isCurrentDate: boolean;
                };
            }[];
        }[];
    };
    navigation: {
        toNext: () => void;
        toPrev: () => void;
        setToday: () => void;
        setDate: (date: Date) => void;
    };
    view: {
        type: CalendarViewType;
        setViewType: import("react").Dispatch<import("react").SetStateAction<CalendarViewType>>;
        setWeekStartsOn: import("react").Dispatch<import("react").SetStateAction<WeekDayType>>;
        isMonthView: boolean;
        isWeekView: boolean;
        isDayView: boolean;
        showMonthView: () => void;
        showWeekView: () => void;
        showDayView: () => void;
    };
};
